declare module 'locators/common' {
	import Q = require("q");
	export interface Location {
	    host: string;
	    port?: number;
	}
	export interface ReturnedLocation {
	    address: string;
	    port: number;
	}
	export interface DataExtractor {
	    (data: string): any;
	}
	export interface Locator {
	    (): Q.Promise<Location>;
	    addListener?(event: string, listener: Function): any;
	    on?(event: string, listener: Function): any;
	    once?(event: string, listener: Function): any;
	    removeListener?(event: string, listener: Function): any;
	    removeAllListeners?(event?: string): any;
	    setMaxListeners?(n: number): void;
	    listeners?(event: string): Function[];
	    emit?(event: string, ...args: any[]): boolean;
	}

}
declare module 'locators/simple' {
	export interface SimpleLocatorParameters {
	    resource: string;
	    defaultPort?: number;
	}
	export function simpleLocatorFactory(): Function;

}
declare module 'locators/locatorException' {
	export var CODE: any;
	export function create(code: string): Error;

}
declare module 'locators/request' {
	import { DataExtractor } from 'locators/common';
	export interface RequestLocatorParameters {
	    url: string;
	    dataExtractor?: DataExtractor;
	}
	export function requestLocatorFactory(): Function;

}
declare module 'locators/zookeeper' {
	import { Locator, DataExtractor } from 'locators/common';
	export interface ZookeeperLocatorParameters {
	    serverLocator: Locator;
	    path: string;
	    dataExtractor?: DataExtractor;
	    locatorTimeout?: number;
	    sessionTimeout?: number;
	    spinDelay?: number;
	    retries?: number;
	    strict?: boolean;
	}
	export function zookeeperLocatorFactory(parameters: ZookeeperLocatorParameters): Function;

}
declare module 'locators/index' {
	import { simpleLocatorFactory } from 'locators/simple';
	import { requestLocatorFactory } from 'locators/request';
	import { zookeeperLocatorFactory } from 'locators/zookeeper';
	import { CODE } from 'locators/locatorException';
	export var simple: typeof simpleLocatorFactory;
	export var request: typeof requestLocatorFactory;
	export var zookeeper: typeof zookeeperLocatorFactory;
	export var EXCEPTION_CODE: typeof CODE;

}
declare module 'locators' {
	import main = require('locators/index');
	export = main;
}
