var Promise = require("q");
var http = require("http");
var https = require("https");
var LocatorException = require("./locatorException");
function defaultDataExtractor(data) {
    var locations;
    try {
        locations = JSON.parse(data).servers;
    }
    catch (e) {
        return null;
    }
    if (!locations) {
        return null;
    }
    var location = locations[Math.floor(Math.random() * locations.length)];
    return {
        host: location.address,
        port: location.port
    };
}
function requestLocatorFactory() {
    function resourceLocator(parameters) {
        if (typeof parameters === "string") {
            parameters = { url: parameters };
        }
        var url = parameters.url;
        var request;
        if (url.indexOf('http://') === 0) {
            request = http;
        }
        else if (url.indexOf('https://') === 0) {
            request = https;
        }
        else {
            throw new Error("invalid url: " + url);
        }
        var dataExtractor = parameters.dataExtractor;
        dataExtractor = dataExtractor ? dataExtractor : defaultDataExtractor;
        if (!url) {
            throw new Error("must have resource");
        }
        var deferred = Promise.defer();
        request.get(url, function (res) {
            var output = [];
            res.setEncoding('utf8');
            res.on('data', function (chunk) {
                output.push(chunk);
            });
            res.on('end', function () {
                if (200 <= res.statusCode && res.statusCode < 300) {
                    var result = output.join('');
                    deferred.resolve(dataExtractor(result));
                }
                else {
                    deferred.reject(LocatorException.create("BAD_RESPONSE"));
                }
            });
        }).on('error', function (err) {
            deferred.reject(err);
        });
        return function () { return deferred.promise; };
    }
    return resourceLocator;
}
exports.requestLocatorFactory = requestLocatorFactory;
